var util = require('util');
var _ = require('underscore');
var Service = require('./service');

var FLAG = require('./flag');

function Client() {
    Service.apply(this, arguments);

    // if run rpc request frequently, 
    // should let this array to like-array object
    // but now, i dont know her
    this.rpcs = [];
}

util.inherits(Client, Service);

Client.prototype.rpcRequest = function (serviceName, body, success, error, timeout) {
    var _me = this;
    var msg = this._genMsg(serviceName, {flag: FLAG.RPC});
    var hasErrorFn = typeof error == "function";
    Service.prototype.rpcRequest.call(this, body, msg);

    if (!hasErrorFn) {
        timeout = error;
        error = null;
    }

    var timer = setTimeout(function () {
        var askId = msg.askId;
        var index = _.findIndex(_me.rpcs, _.matches({askId: askId}));
        
        // in reality, 'index' should be always unequal to -1
        // but in case, we judge it for keeping away from error
        if(index == -1){
            return;
        }

        _me.rpcs.splice(index, 1);
        hasErrorFn && error({
            error: new Error('timeout'),
            serviceName: serviceName,
            body: body,
            askId: askId,
            msg: null
        });
    }, timeout || 1500);

    this.rpcs.push({
        askId: msg.askId,
        serviceName: serviceName,
        success: success,
        error: error,
        timer: timer
    });
};

Client.prototype.destroy = function () {
    Service.prototype.destroy.call(this);
    this.rpcs = [];
};

Client.prototype._onMessage = function (msg) {
    var isCalled = Service.prototype._onMessage.call(this, msg);
    if (isCalled) {
        return;
    }
    var askId = msg.askId;

    // underscore version > 1.7+, it has findIndex method
    // _.matches is already deprecated, but underscore source code still use it
    var index = _.findIndex(this.rpcs, _.matches({askId: askId}));
    var rpcInfo = null;
    if (index == -1) {
        return;
    }else{
        rpcInfo = this.rpcs.splice(index, 1)[0];
    }
    try{
        if(msg.body){
            msg.body = JSON.parse(msg.body.toString());
        }
        var flag = msg.flag;
        if (flag == FLAG.RPC_RES || flag == (FLAG.RPC_RES | FLAG.RPC)) {
            clearTimeout(rpcInfo.timer);
            if (msg.code == 0) {
                rpcInfo.success(msg.body, msg);
            } else if (rpcInfo.error) {
                rpcInfo.error({
                    error: new Error('error'),
                    serviceName: rpcInfo.serviceName,
                    body: msg.body,
                    askId: rpcInfo.askId,
                    msg: msg
                });
            }
        }
    }catch(e){
        rpcInfo.error && rpcInfo.error({
            error: e,
            serviceName: rpcInfo.serviceName,
            body: msg.body,
            askId: rpcInfo.askId,
            msg: msg
        });
    }
};

module.exports = Client;

