var util = require('util');
var crc32 = require('buffer-crc32');
var _ = require('underscore');
var Service = require('./service');
var Response = require('./response');

function Server() {
    Service.apply(this, arguments);
    this.accepts = {};
    this.acceptMap = {};
}

util.inherits(Server, Service);

Server.prototype.addHandler = function (serviceName, callback, isBroadCast, isForce) {
    this.acceptMap[crc32.unsigned(serviceName)] = serviceName;
    this.accepts[serviceName] = {
        callback: _.isFunction(callback) ? callback : noop,
        broadcast: isBroadCast || false,
        force: isForce || false
    };
};

Server.prototype.removeHandler = function (serviceName) {
    var crc32Name = crc32.unsigned(serviceName);
    this.acceptMap[crc32Name] = null;
    this.accepts[serviceName] = null;
    delete this.acceptMap[crc32Name];
    delete this.accepts[serviceName];
};

Server.prototype.destroy = function () {
    Service.prototype.destroy.call(this);
    this.accepts = {};
    this.acceptMap = {};
};

Server.prototype._onMessage = function (msg) {
    var isCalled = Service.prototype._onMessage.call(this, msg);
    if (isCalled) {
        return;
    }

    try{
        msg.body = JSON.parse(msg.body.toString());
        var serviceName = this.acceptMap[msg.service];
        if (serviceName == null) {
            return;
        }

        var accept = this.accepts[serviceName];
        accept.callback(new Response(msg, this));
    }catch(e){
        console.error(e);
        this.emit('error',e);
    }
};

function noop() {

}

module.exports = Server;