var assert = require('assert');
var Client = require('../fpc').Client;
var Server = require('../fpc').Server;
var crc32 = require('buffer-crc32');

var SERVICE_HOST = '172.16.90.42';
var SERVICE_PORT = 9900;

describe('Client', function () {
    var index = 0;
    var serviceName = '';
    this.timeout(20 * 1000);

    beforeEach(function () {
        serviceName = 'unit_test_' + (index++);
    });

    it('connect', function (done) {
        var client = new Client(SERVICE_HOST, SERVICE_PORT);
        client.on('connect', function () {
            done();
        });
    });

    it('register', function (done) {
        var client = new Client(SERVICE_HOST, SERVICE_PORT);
        client.on('connect', function () {
            client.register();
        });
        client.on('register', function () {
            done();
        });
    });

    it('eventRequest', function (done) {
        var server = new Server(SERVICE_HOST, SERVICE_PORT, serviceName);
        server.addHandler(serviceName, function (res) {
            var body = res.msg.body;
            if (body.name == serviceName) {
                done();
            }
        });

        server.on('connect', function () {
            server.register();
        });

        server.on('register', function () {
            var client = new Client(SERVICE_HOST, SERVICE_PORT);
            client.on('connect', function () {
                client.register();
            });

            client.on('register', function () {
                client.eventRequest(serviceName, {
                    name: serviceName
                });
            });
        });
    });

    it('rpcRequest', function (done) {
        var server = new Server(SERVICE_HOST, SERVICE_PORT, serviceName);
        server.addHandler(serviceName, function (res) {
            res.send({
                name: 'unit_test_1'
            });
        });

        server.on('connect', function () {
            server.register();
        });

        server.on('register', function () {
            var client = new Client(SERVICE_HOST, SERVICE_PORT, serviceName);
            client.on('connect', function () {
                client.register();
            });

            client.on('register', function () {
                client.rpcRequest(serviceName, {
                    name: serviceName
                }, function (body) {
                    if (body.name == "unit_test_1") {
                        done();
                    }
                });
            });
        });
    });

    it('rpcRequest　timeout', function (done) {
        var server = new Server(SERVICE_HOST, SERVICE_PORT, serviceName);
        server.addHandler(serviceName, function (res) {
            setTimeout(function () {
                res.send({
                    name: 'unit_test_1'
                });
            }, 2000);
        });

        server.on('connect', function () {
            server.register();
        });

        server.on('register', function () {
            var client = new Client(SERVICE_HOST, SERVICE_PORT, serviceName);
            client.on('connect', function () {
                client.register();
            });

            client.on('register', function () {
                client.rpcRequest(serviceName, {
                    name: serviceName
                }, function () {

                }, function () {
                    done();
                }, 2000);
            });
        });
    });
});