var assert = require('assert');
var ClientPool = require('../fpc').ClientPool;
var Client = require('../fpc').Client;

var SERVICE_HOST = '172.16.90.42';
var SERVICE_PORT = 9900;

describe('ClientPool', function () {
    this.timeout(20 * 1000);

    it('create client pool', function (done) {
        var clientPool = new ClientPool({
            host: SERVICE_HOST,
            port: SERVICE_PORT,
            name: 'unit_test'
        });

        clientPool.on('ready', function () {
            done();
        });
    });

    it('before emit ready, can pass fn', function (done) {
        var clientPool = new ClientPool({
            host: SERVICE_HOST,
            port: SERVICE_PORT,
            name: 'unit_test'
        });

        clientPool.acquire(function (client) {
            if (client instanceof Client) {
                done();
            }
        });
    });

    it('when queue length lt useable client, can cache to queue later to emit', function (done) {
        var clientPool = new ClientPool({
            host: SERVICE_HOST,
            port: SERVICE_PORT,
            name: 'unit_test',
            num: 2
        });

        clientPool.on('ready', function () {
            for (var i = 0; i < 4; i++) {
                (function (index) {
                    clientPool.acquire(function (client) {
                        if (index >= 3 && client instanceof Client) {
                            done();
                        } else {
                            clientPool.release(client);
                        }
                    });
                }(i));
            }
        });
    });

    it('heartbeat can change', function (done) {
        var clientPool = new ClientPool({
            host: SERVICE_HOST,
            port: SERVICE_PORT,
            name: 'unit_test'
        });

        var times = clientPool._poolObj.map(function (client) {
            return client.heartbeat;
        });

        setTimeout(function () {
            assert.notDeepStrictEqual(times, clientPool._poolObj.map(function (client) {
                return client.heartbeat;
            }));
            done();
        }, 15 * 1000);
    });
});