var assert = require("assert");
var _ = require('underscore');
var crc32 = require('buffer-crc32');
var Buffer = require('buffer').Buffer;
var Encoder = require('../encoder');
var Decoder = require('../decoder');

describe("Decoder", function () {
    var msg = {};
    var encoder = null;
    var buffer = null;
    var decoder = null;

    beforeEach(function () {
        msg = {
            service: crc32.unsigned("Container.Common.Ping"),
            flag: 8,
            time: Math.ceil(+new Date() / 1000),
            uniqid: 1,
            askId: 1,
            code: 6000,
            routers: 2,
            dstMode: 1,
            routersList: [new Buffer(8), new Buffer(8)],
            dstParam: new Buffer(8),
            body: {
                name: 'UnitTest'
            }
        };

        encoder = new Encoder();
        buffer = encoder.encode(msg.body, _.omit(msg, 'body'));

        decoder = new Decoder();
    });

    it('decode method -- whole packet', function () {
        decoder.on('message', function (data) {
            decoder.removeAllListeners('message');
            for (var key in msg) {
                assert.deepEqual(msg[key], data[key]);
            }
        });

        decoder.decode(buffer);
    });

    it('decode method -- child packet', function () {
        decoder.on('message', function (data) {
            decoder.removeAllListeners('message');
            for (var key in msg) {
                assert.deepEqual(msg[key], data[key]);
            }
        });

        var b1 = buffer.slice(0, 20);
        var b2 = buffer.slice(20, 40);
        var b3 = buffer.slice(40);
        decoder.decode(b1);
        decoder.decode(b2);
        decoder.decode(b3);
    });

    it('decode method -- adhesion packet', function () {
        buffer = Buffer.concat([buffer, encoder.encode(msg.body, _.omit(msg, 'body'))]);
        decoder.on('message', function (data) {
            for (var key in msg) {
                assert.deepEqual(msg[key], data[key]);
            }
        });

        var b1 = buffer.slice(0, 40);
        var b2 = buffer.slice(40, 70);
        var b3 = buffer.slice(70);
        decoder.decode(b1);
        decoder.decode(b2);
        decoder.decode(b3);
    });
});